import * as React from 'react';
import { Table, Button, Header, Container, Menu, Icon } from 'semantic-ui-react';
import { ListContainer } from '@overflow/commons/react/component/ListContainer';
import Probe from '@overflow/probe/api/model/Probe';
import Domain from '@overflow/domain/api/model/Domain';
import History from '@overflow/history/api/model/History';
import Page from '@overflow/commons/api/model/Page';
import MetaHistoryType from '@overflow/meta/api/model/MetaHistoryType';
import { Pager } from '@overflow/commons/react/component/Pager';


export interface StateProps {
  probes: Probe[];
  histories: Page;
  path: any;
  historyTypes: MetaHistoryType[];
}

export interface DispatchProps {
  onReadAllProbeByDomain?(domain: Domain): void;
  onReadAllMetaHistoryType?(): void;

  onReadAllByDomain?(domain: Domain, pageNo: string, countPerPage: string): void;
  onReadAllByProbe?(probe: Probe, pageNo: string, countPerPage: string): void;
  onReadAllByProbeAndType?(probe: Probe, type: MetaHistoryType, pageNo: string, countPerPage: string): void;
  onReadAllByDomainAndType?(domain: Domain, type: MetaHistoryType, pageNo: string, countPerPage: string): void;
}

export type Props = StateProps & DispatchProps;

export interface State {
  isTypedHistory: boolean;
}

export class HistoryList extends React.Component<Props, State> {

  private countPerPage: number = 10;
  private selectedProbeId: number = 0;
  private selectedTypeId: number = 0;

  constructor(props: Props, context: State) {
    super(props, context);
    this.state = {
      isTypedHistory: false,
    };
  }

  public componentWillMount(): void {
    let domain: Domain = {
      id: 1,
    };
    this.props.onReadAllProbeByDomain(domain);
    this.props.onReadAllMetaHistoryType();

    if (this.props.path === '/histories') {
      this.setState({
        isTypedHistory: false,
      });
      this.getAllHistory(0);
    } else {
      this.setState({
        isTypedHistory: true,
      });
      this.getTypedHistory(0);
    }
  }

  public getAllHistory(pageNo: number): void {
    if (this.selectedProbeId === 0) {
      let domain: Domain = {
        id: 1,
      };
      this.props.onReadAllByDomain(domain, String(pageNo), String(this.countPerPage));
    } else {
      let probe: Probe = {
        id: this.selectedProbeId,
      };
      this.props.onReadAllByProbe(probe, String(pageNo), String(this.countPerPage));
    }
  }

  public getTypedHistory(pageNo: number): void {
    let probe: Probe = {
      id: this.selectedProbeId,
    };
    let type = this.props.path.split('/')[1];
    this.props.onReadAllByProbeAndType(probe, this.getMetaType(type), String(pageNo), String(this.countPerPage));
  }

  public getMetaType(typeStr: string): MetaHistoryType {
    let metaType: MetaHistoryType = {
    };
    if (typeStr === 'member') {
      metaType.id = 1;
    } else if (typeStr === 'probe') {
      metaType.id = 2;
    } else if (typeStr === 'discovery') {
      metaType.id = 3;
    } else if (typeStr === 'target') {
      metaType.id = 4;
    } else if (typeStr === 'crawler') {
      metaType.id = 5;
    } else if (typeStr === 'sensor') {
      metaType.id = 6;
    }
    return metaType;
  }

  public handlePaging = (pageNo: number) => {
    let probe: Probe = {
      id: Number(1),
    };
    if (this.state.isTypedHistory) {
      this.getTypedHistory(pageNo);
    } else {
      this.getAllHistory(pageNo);
    }
  }

  public render(): JSX.Element {
    let historyList: JSX.Element = (
      <Container fluid>
        <Table celled selectable striped>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No.</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Probe</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Type</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Message</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created At</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created By</Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {this.props.histories && this.props.histories.content ?
              this.props.histories.content.map((history: History, index: number) => (
                <Table.Row key={index} >
                  <Table.Cell textAlign={'center'} collapsing>{history.id}</Table.Cell>
                  <Table.Cell textAlign={'center'} collapsing>
                    {history.probe.displayName}
                  </Table.Cell>
                  <Table.Cell textAlign={'center'} collapsing>
                    {history.type.name}
                  </Table.Cell>
                  <Table.Cell>
                    {history.message}
                  </Table.Cell>
                  <Table.Cell collapsing>
                    {history.createDate}
                  </Table.Cell>
                  <Table.Cell collapsing>
                    {history.member.name}
                  </Table.Cell>
                </Table.Row>
              )) : ''}
          </Table.Body>
          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='3'>
                <Pager page={this.props.histories} onPageChange={this.handlePaging} />
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table>
      </Container>
    );

    let filter = <HistoryFilter probes={this.props.probes}
      onProbeChange={this.handleProbeChange}
      historyTypes={this.props.historyTypes}
      onTypeChange={this.handleTypeChange}
    />;
    return (
      <ListContainer
        filter={filter}
        contents={historyList}
        data={this.props.histories}
      />
    );
  }

  // todo. refactoring
  private handleProbeChange = (probeId: number) => {
    this.selectedProbeId = probeId;
    this.getHistoryByFilter();
  }
  // todo. refactoring
  private handleTypeChange = (typeId: number) => {
    this.selectedTypeId = typeId;
    this.getHistoryByFilter();
  }
  // todo. refactoring
  private getHistoryByFilter = () => {
    if (this.selectedProbeId === 0) {
      // readAllByDomain or readAllByDomainAndType
      const domain: Domain = {
        id: 1,
      };
      if (this.selectedTypeId === 0) {
        this.props.onReadAllByDomain(domain, String(0), String(this.countPerPage));
      } else {
        const type: MetaHistoryType = {
          id: this.selectedTypeId,
        };
        this.props.onReadAllByDomainAndType(domain, type, String(0), String(this.countPerPage));
      }
    } else {
      // readAllByProbe or readAllByProbeAndType
      const probe: Probe = {
        id: this.selectedProbeId,
      };
      if (this.selectedTypeId === 0) {
        this.props.onReadAllByProbe(probe, String(0), String(this.countPerPage));
      } else {
        const type: MetaHistoryType = {
          id: this.selectedTypeId,
        };
        this.props.onReadAllByProbeAndType(probe, type, String(0), String(this.countPerPage));
      }
    }
  }
}




export interface FilterProps {
  probes: Probe[];
  historyTypes: MetaHistoryType[];
  onProbeChange(probeId: number): void;
  onTypeChange(typeId: number): void;
}

export interface FilterState {
  selectedProbeId: number;
  selectedTypeId: number;
}

import { Dropdown, Form, Radio } from 'semantic-ui-react';

export class HistoryFilter extends React.Component<FilterProps, FilterState> {

  constructor(props: any, context: any) {
    super(props, context);
    this.state = {
      selectedProbeId: 0,
      selectedTypeId: 0,
    };
  }

  public render(): JSX.Element {
    if (this.props.probes === null || this.props.probes === undefined) {
      return null;
    }
    return (
      <Form>
        <Form.Field>Probe</Form.Field>
        <Form.Field>
          <Dropdown selection placeholder='All' options={this.getOptions()}
            onChange={(event: React.SyntheticEvent<HTMLElement>, data: any) => {
              this.setState({
                selectedProbeId: data.value,
              });
              this.props.onProbeChange(data.value);
            }} />
        </Form.Field>
        <Form.Field>Type</Form.Field>
        {this.renderTypesRadio()}

      </Form>
    );
  }

  private renderTypesRadio = (): JSX.Element[] => {
    if (this.props.historyTypes === undefined) {
      return null;
    }
    return this.props.historyTypes.map((type: MetaHistoryType, index: number) => (
      <Form.Field key={index}>
        <Radio
          slider
          label={type.name}
          name='historyType'
          value={type.id}
          checked={this.state.selectedTypeId === type.id}
          onChange={this.handleTypeChange}
        />
      </Form.Field>
    ));
  }

  private handleTypeChange = (e: React.SyntheticEvent<HTMLInputElement>, data: any) => {
    this.setState({ selectedTypeId: data.value });
    this.props.onTypeChange(data.value);
  }

  private getOptions(): Array<object> {
    let res = new Array;
    let all = {
      key: 0,
      text: 'All',
      value: 0,
    };
    res.push(all);
    this.props.probes.map((probe: Probe, index: number) => {
      let option = {
        key: index + 1,
        text: probe.displayName,
        value: probe.id,
      };
      res.push(option);
    });
    return res;
  }
}
